@@ -67,7 +67,7 @@ struct _GKMKeyringEditorPrivate
 
   gboolean is_multiline_secret;
 
-  GtkWidget *keyring_items_context_menu;
+  GtkUIManager *ui_manager;
 
   GtkListStore *keyring_items;
 
@@ -110,11 +110,6 @@ static void gkm_keyring_editor_finalize   (GObject *object);
 static void gkm_keyring_editor_destroy    (GtkObject *object);
 
 const char *keyring_item_type_text_from_enum (GnomeKeyringItemType type);
-static void format_keyring_item_type_as_text (GtkTreeViewColumn *column, 
-                                              GtkCellRenderer *renderer, 
-                                              GtkTreeModel *model,
-                                              GtkTreeIter *iter, 
-                                              gpointer *data);
 static void gkm_keyring_editor_setup_keyring_items_tree (GKMKeyringEditor *editor);
 
 static void open_keyring_manager_callback (GtkWidget *widget, GKMKeyringEditor *editor);
@@ -259,7 +254,6 @@ gkm_keyring_editor_init (GKMKeyringEditor *editor)
   GtkWidget *alignment;
   GtkSizeGroup *button_sizegroup;
   GtkTooltips *tooltips;
-  GtkUIManager *ui_manager;
   GtkAccelGroup *accel_group;
   GtkActionGroup *action_group;
   GdkPixbuf *keyring_pixbuf;
@@ -281,26 +275,22 @@ gkm_keyring_editor_init (GKMKeyringEditor *editor)
   gtk_action_group_add_actions (action_group, entries, G_N_ELEMENTS (entries), editor);
   gtk_action_group_add_toggle_actions (action_group, toggle_entries, G_N_ELEMENTS (toggle_entries), editor);
   
-  ui_manager = gtk_ui_manager_new ();
-  gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
+  editor->priv->ui_manager = gtk_ui_manager_new ();
+  gtk_ui_manager_insert_action_group (editor->priv->ui_manager, action_group, 0);
 
-  accel_group = gtk_ui_manager_get_accel_group (ui_manager);
+  accel_group = gtk_ui_manager_get_accel_group (editor->priv->ui_manager);
   gtk_window_add_accel_group (GTK_WINDOW (editor), accel_group);
 
   error = NULL;
-  if (!gtk_ui_manager_add_ui_from_string (ui_manager, ui_description, -1, &error))
+  if (!gtk_ui_manager_add_ui_from_string (editor->priv->ui_manager, ui_description, -1, &error))
     {
       g_message ("Building menus failed: %s", error->message);
       g_error_free (error);
       exit (1);
     }
 
-  menubar = gtk_ui_manager_get_widget (ui_manager, "/KeyringEditorMenu");
+  menubar = gtk_ui_manager_get_widget (editor->priv->ui_manager, "/KeyringEditorMenu");
   gtk_box_pack_start (GTK_BOX (main_vbox), menubar, FALSE, FALSE, 0);
-
-  editor->priv->keyring_items_context_menu = gtk_ui_manager_get_widget (ui_manager, "/KeyringItemContextMenu");
-
-  g_object_unref (ui_manager);
   
   groups_vbox = gtk_vbox_new (FALSE, 18);
   gtk_container_set_border_width (GTK_CONTAINER (groups_vbox), 12);
@@ -502,6 +492,8 @@ gkm_keyring_editor_finalize (GObject *object)
   g_free (editor->priv->keyring_name);
 
   close_gconf_connections (editor->priv->gconf_cnxn_ids, GKM_KEYRING_EDITOR_GCONF_DIR, gconf_client);
+  
+  g_object_unref (G_OBJECT (editor->priv->ui_manager));
  
   g_free (editor->priv);
 
@@ -599,18 +591,29 @@ keyring_item_type_text_from_enum (GnomeKeyringItemType type)
 }
 
 static void 
-format_keyring_item_type_as_text (GtkTreeViewColumn *column G_GNUC_UNUSED, 
-                                  GtkCellRenderer *renderer, 
-                                  GtkTreeModel *model,
-                                  GtkTreeIter *iter, 
-                                  gpointer *data G_GNUC_UNUSED)
+keyring_item_type_get_pixbuf (GtkTreeViewColumn *column G_GNUC_UNUSED, 
+                              GtkCellRenderer *renderer, 
+                              GtkTreeModel *model,
+                              GtkTreeIter *iter, 
+                              gpointer *data G_GNUC_UNUSED)
 {
   int type;
-  const char *text;
 
   gtk_tree_model_get (model, iter, ITEM_COLUMN_TYPE, &type, -1);
-  text = keyring_item_type_text_from_enum (type);
-  g_object_set (G_OBJECT (renderer), "text", text, NULL);
+  switch (type)
+    {
+      case GNOME_KEYRING_ITEM_GENERIC_SECRET:
+        g_object_set (G_OBJECT (renderer), "stock-id", GNOME_STOCK_AUTHENTICATION, NULL);
+        break;
+      case GNOME_KEYRING_ITEM_NETWORK_PASSWORD:
+        g_object_set (G_OBJECT (renderer), "stock-id", GTK_STOCK_NETWORK, NULL);
+        break;
+      case GNOME_KEYRING_ITEM_NOTE:
+        g_object_set (G_OBJECT (renderer), "stock-id", GNOME_STOCK_BOOK_OPEN, NULL);
+        break;
+      default:
+        g_object_set (G_OBJECT (renderer), "stock-id", GNOME_STOCK_BLANK, NULL);
+    }
 }
 
 static void
@@ -625,17 +628,17 @@ gkm_keyring_editor_setup_keyring_items_tree (GKMKeyringEditor *editor)
   editor->priv->keyring_items = gtk_list_store_new (NUM_ITEM_COLUMNS, 
                                                     /* ITEM_COLUMN_NAME */   G_TYPE_STRING, 
                                                     /* ITEM_COLUMN_TYPE */   G_TYPE_INT, 
-                                                    /* ITEM_COLUMN_MTIME */  G_TYPE_STRING, 
-                                                    /* ITEM_COLUMN_CTIME */  G_TYPE_STRING, 
+                                                    /* ITEM_COLUMN_MTIME */  G_TYPE_UINT, 
+                                                    /* ITEM_COLUMN_CTIME */  G_TYPE_UINT, 
                                                     /* ITEM_COLUMN_ID */     G_TYPE_UINT, 
                                                     /* ITEM_COLUMN_SECRET */ G_TYPE_STRING);
   
   column = gtk_tree_view_column_new ();
   gtk_tree_view_column_set_title (column, _("Type"));
   gtk_tree_view_append_column (GTK_TREE_VIEW (editor->priv->items_tree_view), column);
-  renderer = gtk_cell_renderer_text_new ();
+  renderer = gtk_cell_renderer_pixbuf_new ();
   gtk_tree_view_column_pack_start (column, renderer, TRUE);
-  gtk_tree_view_column_set_cell_data_func (column, renderer, (GtkTreeCellDataFunc) format_keyring_item_type_as_text, NULL, NULL);
+  gtk_tree_view_column_set_cell_data_func (column, renderer, (GtkTreeCellDataFunc) keyring_item_type_get_pixbuf, NULL, NULL);
   gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (editor->priv->keyring_items), ITEM_COLUMN_TYPE, 
                                    (GtkTreeIterCompareFunc) compare_keyring_item_types, GINT_TO_POINTER (ITEM_COLUMN_TYPE), NULL);
   gtk_tree_view_column_set_sort_column_id (column, ITEM_COLUMN_TYPE);
@@ -647,6 +650,10 @@ gkm_keyring_editor_setup_keyring_items_tree (GKMKeyringEditor *editor)
   renderer = gtk_cell_renderer_text_new ();
   gtk_tree_view_column_pack_start (column, renderer, TRUE);
   gtk_tree_view_column_add_attribute (column, renderer, "text", ITEM_COLUMN_NAME);
+  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (editor->priv->keyring_items),
+  				   ITEM_COLUMN_NAME,
+				   (GtkTreeIterCompareFunc) tree_model_compare_strings,
+				   GINT_TO_POINTER (ITEM_COLUMN_NAME), NULL);
   gtk_tree_view_column_set_sort_column_id (column, ITEM_COLUMN_NAME);
   
   column = gtk_tree_view_column_new ();
@@ -654,16 +661,36 @@ gkm_keyring_editor_setup_keyring_items_tree (GKMKeyringEditor *editor)
   gtk_tree_view_append_column (GTK_TREE_VIEW (editor->priv->items_tree_view), column);
   renderer = gtk_cell_renderer_text_new ();
   gtk_tree_view_column_pack_start (column, renderer, TRUE);
-  gtk_tree_view_column_add_attribute (column, renderer, "text", ITEM_COLUMN_MTIME);
   gtk_tree_view_column_set_visible (column, FALSE);
+  gtk_tree_view_column_set_cell_data_func (column,
+  				           renderer,
+				           (GtkTreeCellDataFunc) format_date_cell_data_func,
+				           GINT_TO_POINTER (ITEM_COLUMN_MTIME), NULL);
+  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (editor->priv->keyring_items),
+  				   ITEM_COLUMN_MTIME,
+				   (GtkTreeIterCompareFunc) tree_model_compare_uints,
+				   GINT_TO_POINTER (ITEM_COLUMN_MTIME), NULL);
+  gtk_tree_view_column_set_sort_column_id (column, ITEM_COLUMN_MTIME);
 
   column = gtk_tree_view_column_new ();
   gtk_tree_view_column_set_title (column, _("Creation Time"));
   gtk_tree_view_append_column (GTK_TREE_VIEW (editor->priv->items_tree_view), column);
   renderer = gtk_cell_renderer_text_new ();
   gtk_tree_view_column_pack_start (column, renderer, TRUE);
-  gtk_tree_view_column_add_attribute (column, renderer, "text", ITEM_COLUMN_CTIME);
   gtk_tree_view_column_set_visible (column, FALSE);
+  gtk_tree_view_column_set_cell_data_func (column,
+  				           renderer,
+				           (GtkTreeCellDataFunc) format_date_cell_data_func,
+				           GINT_TO_POINTER (ITEM_COLUMN_CTIME), NULL);
+  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (editor->priv->keyring_items),
+  				   ITEM_COLUMN_CTIME,
+				   (GtkTreeIterCompareFunc) tree_model_compare_uints,
+				   GINT_TO_POINTER (ITEM_COLUMN_CTIME), NULL);
+  gtk_tree_view_column_set_sort_column_id (column, ITEM_COLUMN_CTIME);
+
+  gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (editor->priv->keyring_items),
+				           (GtkTreeIterCompareFunc) tree_model_compare_uints,
+				           GINT_TO_POINTER (ITEM_COLUMN_ID), NULL);
 
   gtk_tree_view_set_model (GTK_TREE_VIEW (editor->priv->items_tree_view), GTK_TREE_MODEL (editor->priv->keyring_items));
   g_object_unref (G_OBJECT (editor->priv->keyring_items));
@@ -1275,7 +1302,6 @@ delete_item_done_callback (GnomeKeyringResult        result,
           gtk_tree_selection_select_iter (data->selection, &data->iter);
         }
     }
-  gkm_keyring_editor_set_secret (data->editor, "");
 }
 
 static void
@@ -1545,10 +1571,13 @@ keyring_items_tree_popup_menu (GtkWidget        *tree_view,
 			       GdkEventButton   *event,
 			       GKMKeyringEditor *editor)
 {
+  GtkWidget *context_menu;
+  
   g_return_if_fail (GTK_IS_TREE_VIEW (tree_view));
   g_return_if_fail (GKM_IS_KEYRING_EDITOR (editor));
 
-  gtk_menu_popup (GTK_MENU (editor->priv->keyring_items_context_menu), NULL, NULL, NULL, NULL,
+  context_menu = gtk_ui_manager_get_widget (editor->priv->ui_manager, "/KeyringItemContextMenu");
+  gtk_menu_popup (GTK_MENU (context_menu), NULL, NULL, NULL, NULL,
   		  (event != NULL) ? event->button : 0,
 		  gdk_event_get_time ( (GdkEvent *) event));
 }
@@ -1666,6 +1695,7 @@ keyring_item_tree_selection_changed (GtkTreeSelection  *selection,
   else
     {
       keyring_editor_update_sensitivity (editor, FALSE);
+      gkm_keyring_editor_set_secret (editor, "");
     }
 }
 
@@ -1872,14 +1902,10 @@ update_keyring_item_info_worker_callback (GnomeKeyringResult             result,
   g_free (value);
 
   time = gnome_keyring_item_info_get_ctime (info);
-  value = format_date_for_display (time);
-  gtk_list_store_set (data->editor->priv->keyring_items, &iter, ITEM_COLUMN_CTIME, value, -1);
-  g_free (value);
+  gtk_list_store_set (data->editor->priv->keyring_items, &iter, ITEM_COLUMN_CTIME, time, -1);
 
   time = gnome_keyring_item_info_get_mtime (info);
-  value = format_date_for_display (time);
-  gtk_list_store_set (data->editor->priv->keyring_items, &iter, ITEM_COLUMN_MTIME, value, -1);
-  g_free (value);
+  gtk_list_store_set (data->editor->priv->keyring_items, &iter, ITEM_COLUMN_MTIME, time, -1);
 }
 
 /**
@@ -1906,7 +1932,7 @@ gkm_keyring_editor_update_keyring_item_info (GKMKeyringEditor *editor,
     {
       if (!search_keyring_item (editor, item_id, NULL, NULL, &data->row))
         {
-          g_warning (_("gkm_keyring_editor_update_keyring_item_info: Tried to update the information of a keyring item we don't know about"));
+          g_warning ("gkm_keyring_editor_update_keyring_item_info: Tried to update the information of a keyring item we don't know about");
         }
     }
   else
