@@ -71,7 +71,7 @@ struct _GKMKeyringManagerPrivate
   GtkWidget *create_dialog;
   GtkWidget *unlock_dialog;
 
-  GtkWidget *keyring_context_menu;
+  GtkUIManager *ui_manager;
 
   GSList *gconf_cnxn_ids;
 };
@@ -234,11 +234,9 @@ gkm_keyring_manager_init (GKMKeyringManager *manager)
   GtkWidget *vbuttonbox;
   GtkWidget *button;
   GtkWidget *statusbar;
-  GtkUIManager *ui_manager;
   GtkAccelGroup *accel_group;
   GtkActionGroup *action_group;
   GtkWidget *menubar;
-  GtkWidget *menu_item;
   GError *error;
 
   manager->priv = g_new0 (GKMKeyringManagerPrivate, 1);
@@ -253,32 +251,25 @@ gkm_keyring_manager_init (GKMKeyringManager *manager)
   gtk_action_group_add_actions (action_group, entries, G_N_ELEMENTS (entries), manager);
   gtk_action_group_add_toggle_actions (action_group, toggle_entries, G_N_ELEMENTS (toggle_entries), manager);
   
-  ui_manager = gtk_ui_manager_new ();
-  gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
+  manager->priv->ui_manager = gtk_ui_manager_new ();
+  gtk_ui_manager_insert_action_group (manager->priv->ui_manager,
+                                      action_group, 0);
 
-  accel_group = gtk_ui_manager_get_accel_group (ui_manager);
+  accel_group = gtk_ui_manager_get_accel_group (manager->priv->ui_manager);
   gtk_window_add_accel_group (GTK_WINDOW (manager), accel_group);
 
   error = NULL;
-  if (!gtk_ui_manager_add_ui_from_string (ui_manager, ui_description, -1, &error))
+  if (!gtk_ui_manager_add_ui_from_string (manager->priv->ui_manager,
+                                          ui_description, -1, &error))
     {
       g_message ("building menus failed: %s", error->message);
       g_error_free (error);
       exit (1);
     }
 
-  menubar = gtk_ui_manager_get_widget (ui_manager, "/KeyringManagerMenu");
+  menubar = gtk_ui_manager_get_widget (manager->priv->ui_manager, "/KeyringManagerMenu");
   gtk_box_pack_start (GTK_BOX (main_vbox), menubar, FALSE, FALSE, 0);
 
-  manager->priv->keyring_context_menu = gtk_ui_manager_get_widget (ui_manager, "/KeyringContextMenu");
-  menu_item = gtk_ui_manager_get_widget (ui_manager, "/KeyringContextMenu/LockKeyring");
-  g_object_set_data (G_OBJECT (manager->priv->keyring_context_menu), "lock-item", menu_item);
-  menu_item = gtk_ui_manager_get_widget (ui_manager, "/KeyringContextMenu/UnlockKeyring");
-  g_object_set_data (G_OBJECT (manager->priv->keyring_context_menu), "unlock-item", menu_item);
-  menu_item = gtk_ui_manager_get_widget (ui_manager, "/KeyringContextMenu/MakeDefault");
-  g_object_set_data (G_OBJECT (manager->priv->keyring_context_menu), "make-default-item", menu_item);
-  
-  g_object_unref (ui_manager);
   hbox = gtk_hbox_new (FALSE, 12);
   gtk_container_set_border_width (GTK_CONTAINER (hbox), 18);
   gtk_box_pack_start (GTK_BOX (main_vbox), hbox, TRUE, TRUE, 0);
@@ -368,6 +359,8 @@ gkm_keyring_manager_finalize (GObject *object)
   close_gconf_connections (manager->priv->gconf_cnxn_ids,
   			   GKM_KEYRING_MANAGER_GCONF_DIR,
 			   gconf_client);
+	
+	g_object_unref (G_OBJECT (manager->priv->ui_manager));
   
   g_free (manager->priv);
 
@@ -431,6 +424,56 @@ gkm_keyring_manager_register_icons (void)
   been_here = TRUE;
 }
 
+static gint
+compare_keyring_lock_status (GtkTreeModel *model,
+			     GtkTreeIter  *a,
+			     GtkTreeIter  *b,
+			     gpointer      data)
+{
+  gint   sortcol;
+  gint   ret;
+  gchar *string1;
+  gchar *string2;
+
+  sortcol = GPOINTER_TO_INT (data);
+  ret = 0;
+ 
+  gtk_tree_model_get (model, a, sortcol, &string1, -1);
+  gtk_tree_model_get (model, b, sortcol, &string2, -1);
+  
+  /* Put Unlocked keyrings first. */
+  if (strcmp (string1, string2) != 0)
+    {
+      ret = (strcmp (string1, UNLOCKED_ICON) == 0) ? -1 : 1;
+    }
+  
+  g_free (string1);
+  g_free (string2);
+  
+  return ret;
+}
+
+static void
+format_timeout_cell_data_func (GtkTreeViewColumn *column G_GNUC_UNUSED,
+                               GtkCellRenderer   *cell,
+                               GtkTreeModel      *model,
+                               GtkTreeIter       *iter,
+                               gpointer           data)
+{
+  gint column_id;
+  guint32 timeout;
+  gchar *readable_timeout;
+
+  column_id = GPOINTER_TO_INT (data);
+
+  gtk_tree_model_get(model, iter, column_id, &timeout, -1); 
+  readable_timeout = g_strdup_printf (_("%d seconds"), timeout);
+  
+  g_object_set(cell, "text", readable_timeout, NULL);
+
+  g_free (readable_timeout);
+}
+
 static void
 gkm_keyring_manager_initialize_tree_view (GKMKeyringManager *manager)
 {
@@ -441,20 +484,26 @@ gkm_keyring_manager_initialize_tree_view (GKMKeyringManager *manager)
   g_assert (manager->priv->keyrings == NULL);
 
   manager->priv->keyrings = gtk_list_store_new (NUM_COLUMNS, 
-                                                G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, 
-                                                G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
+                                                G_TYPE_STRING, G_TYPE_STRING, G_TYPE_UINT, 
+                                                G_TYPE_STRING, G_TYPE_UINT, G_TYPE_UINT,
                                                 G_TYPE_STRING);
 
   column = gtk_tree_view_column_new ();
   gtk_tree_view_column_set_title (column, _("Keyring"));
   gtk_tree_view_append_column (GTK_TREE_VIEW (manager->priv->tree_view), column);
+  gtk_tree_view_column_set_expand (column, TRUE);
   renderer = gtk_cell_renderer_text_new ();
   gtk_tree_view_column_pack_start (column, renderer, TRUE);
   gtk_tree_view_column_add_attribute (column, renderer, "text", COLUMN_KEYRING);
   renderer = gtk_cell_renderer_pixbuf_new ();
   gtk_tree_view_column_pack_start (column, renderer, FALSE);
   gtk_tree_view_column_add_attribute (column, renderer, "stock-id", COLUMN_DEFAULT);
-  
+  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (manager->priv->keyrings),
+  				   COLUMN_KEYRING,
+				   (GtkTreeIterCompareFunc) tree_model_compare_strings,
+				   GINT_TO_POINTER (COLUMN_KEYRING), NULL);
+  gtk_tree_view_column_set_sort_column_id (column, COLUMN_KEYRING);
+
   column = gtk_tree_view_column_new ();
   gtk_tree_view_column_set_title (column, _("Locked"));
   gtk_tree_view_append_column (GTK_TREE_VIEW (manager->priv->tree_view), column);
@@ -462,36 +511,74 @@ gkm_keyring_manager_initialize_tree_view (GKMKeyringManager *manager)
   gtk_tree_view_column_pack_start (column, renderer, TRUE);
   gtk_tree_view_column_add_attribute (column, renderer, "stock-id", COLUMN_LOCK);
   g_object_set (G_OBJECT (renderer), "xalign", 0.0, "xpad", 3, NULL);
-  
+  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (manager->priv->keyrings),
+  				   COLUMN_LOCK,
+				   (GtkTreeIterCompareFunc) compare_keyring_lock_status,
+				   GINT_TO_POINTER (COLUMN_LOCK), NULL);
+  gtk_tree_view_column_set_sort_column_id (column, COLUMN_LOCK);
+ 
   column = gtk_tree_view_column_new ();
   gtk_tree_view_column_set_title (column, _("Lock on Idle"));
   gtk_tree_view_append_column (GTK_TREE_VIEW (manager->priv->tree_view), column);
   renderer = gtk_cell_renderer_text_new ();
   gtk_tree_view_column_pack_start (column, renderer, TRUE);
   gtk_tree_view_column_add_attribute (column, renderer, "text", COLUMN_LOCK_ON_IDLE);
-  
+  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (manager->priv->keyrings),
+  				   COLUMN_LOCK_ON_IDLE,
+				   (GtkTreeIterCompareFunc) tree_model_compare_strings,
+				   GINT_TO_POINTER (COLUMN_LOCK_ON_IDLE), NULL);
+  gtk_tree_view_column_set_sort_column_id (column, COLUMN_LOCK_ON_IDLE);
+ 
   column = gtk_tree_view_column_new ();
   gtk_tree_view_column_set_title (column, _("Lock Timeout"));
   gtk_tree_view_append_column (GTK_TREE_VIEW (manager->priv->tree_view), column);
   renderer = gtk_cell_renderer_text_new ();
   gtk_tree_view_column_pack_start (column, renderer, TRUE);
-  gtk_tree_view_column_add_attribute (column, renderer, "text", COLUMN_LOCK_TIMEOUT);
-  
+  gtk_tree_view_column_set_cell_data_func (column,
+  				           renderer,
+				           (GtkTreeCellDataFunc) format_timeout_cell_data_func,
+				           GINT_TO_POINTER (COLUMN_LOCK_TIMEOUT), NULL);
+  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (manager->priv->keyrings),
+  				   COLUMN_LOCK_TIMEOUT,
+				   (GtkTreeIterCompareFunc) tree_model_compare_uints,
+				   GINT_TO_POINTER (COLUMN_LOCK_TIMEOUT), NULL);
+  gtk_tree_view_column_set_sort_column_id (column, COLUMN_LOCK_TIMEOUT);
+ 
   column = gtk_tree_view_column_new ();
   gtk_tree_view_column_set_title (column, _("Modification Time"));
   gtk_tree_view_append_column (GTK_TREE_VIEW (manager->priv->tree_view), column);
   renderer = gtk_cell_renderer_text_new ();
   gtk_tree_view_column_pack_start (column, renderer, TRUE);
-  gtk_tree_view_column_add_attribute (column, renderer, "text", COLUMN_MTIME);
   gtk_tree_view_column_set_visible (column, FALSE);
-  
+  gtk_tree_view_column_set_cell_data_func (column,
+  				           renderer,
+				           (GtkTreeCellDataFunc) format_date_cell_data_func,
+				           GINT_TO_POINTER (COLUMN_MTIME), NULL);
+  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (manager->priv->keyrings),
+  				   COLUMN_MTIME,
+				   (GtkTreeIterCompareFunc) tree_model_compare_uints,
+				   GINT_TO_POINTER (COLUMN_MTIME), NULL);
+  gtk_tree_view_column_set_sort_column_id (column, COLUMN_MTIME);
+ 
   column = gtk_tree_view_column_new ();
   gtk_tree_view_column_set_title (column, _("Creation Time"));
   gtk_tree_view_append_column (GTK_TREE_VIEW (manager->priv->tree_view), column);
   renderer = gtk_cell_renderer_text_new ();
   gtk_tree_view_column_pack_start (column, renderer, TRUE);
-  gtk_tree_view_column_add_attribute (column, renderer, "text", COLUMN_CTIME);
   gtk_tree_view_column_set_visible (column, FALSE);
+  gtk_tree_view_column_set_cell_data_func (column,
+  				           renderer,
+				           (GtkTreeCellDataFunc) format_date_cell_data_func,
+				           GINT_TO_POINTER (COLUMN_CTIME), NULL);
+  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (manager->priv->keyrings),
+  				   COLUMN_CTIME,
+				   (GtkTreeIterCompareFunc) tree_model_compare_uints,
+				   GINT_TO_POINTER (COLUMN_CTIME), NULL);
+  gtk_tree_view_column_set_sort_column_id (column, COLUMN_CTIME);
+
+  gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (manager->priv->keyrings),
+				           (GtkTreeIterCompareFunc) tree_model_compare_strings,
+				           GINT_TO_POINTER (COLUMN_KEYRING), NULL);
 
   gtk_tree_view_set_model (GTK_TREE_VIEW (manager->priv->tree_view), GTK_TREE_MODEL (manager->priv->keyrings));
   g_object_unref (G_OBJECT (manager->priv->keyrings));
@@ -1060,6 +1147,7 @@ keyring_tree_popup_menu (GtkWidget         *tree_view,
 		         GKMKeyringManager *manager)
 {
   GtkWidget *menu_item;
+  GtkWidget *menu;
   char *state;
  
   g_return_if_fail (GTK_IS_TREE_VIEW (tree_view));
@@ -1068,31 +1156,33 @@ keyring_tree_popup_menu (GtkWidget         *tree_view,
   /* This is getting the state from the keyrings ListStore, which is different from
    * what we do in lock_unlock_keyring_callback...
    */
-
+  
+  menu = gtk_ui_manager_get_widget (manager->priv->ui_manager, "/KeyringContextMenu");
+  
   gtk_tree_model_get (GTK_TREE_MODEL (manager->priv->keyrings), iter, COLUMN_LOCK, &state, -1);
   if (strcmp (state, LOCKED_ICON) == 0)
     {
-      menu_item = g_object_get_data (G_OBJECT (manager->priv->keyring_context_menu), "lock-item");
+      menu_item = gtk_ui_manager_get_widget (manager->priv->ui_manager, "/KeyringContextMenu/LockKeyring");
       gtk_widget_hide (menu_item);
-      menu_item = g_object_get_data (G_OBJECT (manager->priv->keyring_context_menu), "unlock-item");
+      menu_item = gtk_ui_manager_get_widget (manager->priv->ui_manager, "/KeyringContextMenu/UnlockKeyring");
       gtk_widget_show (menu_item);
     }
   else
     {
-      menu_item = g_object_get_data (G_OBJECT (manager->priv->keyring_context_menu), "unlock-item");
+      menu_item = gtk_ui_manager_get_widget (manager->priv->ui_manager, "/KeyringContextMenu/UnlockKeyring");
       gtk_widget_hide (menu_item);
-      menu_item = g_object_get_data (G_OBJECT (manager->priv->keyring_context_menu), "lock-item");
+      menu_item = gtk_ui_manager_get_widget (manager->priv->ui_manager, "/KeyringContextMenu/LockKeyring");
       gtk_widget_show (menu_item);
     }
   g_free (state);
 
 
   gtk_tree_model_get (GTK_TREE_MODEL (manager->priv->keyrings), iter, COLUMN_DEFAULT, &state, -1);
-  menu_item = g_object_get_data (G_OBJECT (manager->priv->keyring_context_menu), "make-default-item");
+  menu_item = gtk_ui_manager_get_widget (manager->priv->ui_manager, "/KeyringContextMenu/MakeDefault");
   gtk_widget_set_sensitive (menu_item, state == NULL);
   g_free (state);
 
-  gtk_menu_popup (GTK_MENU (manager->priv->keyring_context_menu), NULL, NULL, NULL, NULL,
+  gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
   		  (event != NULL) ? event->button : 0,
 		  gdk_event_get_time ( (GdkEvent *) event));
 }
@@ -1328,6 +1418,7 @@ update_keyring_info_worker_callback (GnomeKeyringResult         result,
   char *value;
   const char *stock_id;
   time_t time;
+  guint32 seconds;
   GtkTreePath *path;
   GtkTreeIter iter;
   
@@ -1355,19 +1446,14 @@ update_keyring_info_worker_callback (GnomeKeyringResult         result,
   value = (gnome_keyring_info_get_lock_on_idle (info) == TRUE) ? yes : no;
   gtk_list_store_set (data->manager->priv->keyrings, &iter, COLUMN_LOCK_ON_IDLE, value, -1);
   
-  value = g_strdup_printf (_("%d seconds"), gnome_keyring_info_get_lock_timeout (info));
-  gtk_list_store_set (data->manager->priv->keyrings, &iter, COLUMN_LOCK_TIMEOUT, value, -1);
-  g_free (value);
+  seconds =  gnome_keyring_info_get_lock_timeout (info);
+  gtk_list_store_set (data->manager->priv->keyrings, &iter, COLUMN_LOCK_TIMEOUT, seconds, -1);
   
   time = gnome_keyring_info_get_ctime (info);
-  value = format_date_for_display (time);
-  gtk_list_store_set (data->manager->priv->keyrings, &iter, COLUMN_CTIME, value, -1);
-  g_free (value);
+  gtk_list_store_set (data->manager->priv->keyrings, &iter, COLUMN_CTIME, time, -1);
 
   time = gnome_keyring_info_get_mtime (info);
-  value = format_date_for_display (time);
-  gtk_list_store_set (data->manager->priv->keyrings, &iter, COLUMN_MTIME, value, -1);
-  g_free (value);
+  gtk_list_store_set (data->manager->priv->keyrings, &iter, COLUMN_MTIME, time, -1);
 
   gtk_list_store_set (data->manager->priv->keyrings, &iter, COLUMN_DEFAULT, NULL, -1);
 }
@@ -1396,7 +1482,7 @@ gkm_keyring_manager_update_keyring_info (GKMKeyringManager *manager,
     {
       if (!gkm_keyring_manager_search_keyring (manager, keyring, NULL, NULL, &data->row))
         {
-          g_warning (_("gkm_keyring_manager_update_keyring_info: Tried to update the information of a keyring we don't know about"));
+          g_warning ("gkm_keyring_manager_update_keyring_info: Tried to update the information of a keyring we don't know about");
         }
     }
   else
