@@ -339,7 +339,7 @@ set_column_visibility_preferences (const SetColumnPrefsData *toggle,
       id = gconf_client_notify_add (gconf_client, key, (GConfClientNotifyFunc) update_column_visibility, column, NULL, &error);
       if (error != NULL)
         {
-          g_warning (_("Error while trying to add a listener for changed on the gconf key %s: %s"), key, error->message);
+          g_warning (_("Error while trying to add a listener for changes on the gconf key %s: %s"), key, error->message);
           g_error_free (error);
         }
 
@@ -374,10 +374,83 @@ close_gconf_connections (GSList      *connections,
   gconf_client_remove_dir (gconf_client, dir, &error);
   if (error != NULL)
     {
-      g_printerr (_("Error while trying to remove the %s dir from the list of gconf directories we listen to: %s"), 
+      g_printerr (_("Error while trying to remove the %s directory from the list of gconf directories we listen to: %s"), 
                   dir, error->message);
       g_error_free (error);
     }
   g_free (dir);
 }
 
+void 
+format_date_cell_data_func (GtkTreeViewColumn *column G_GNUC_UNUSED,
+                            GtkCellRenderer   *cell,
+                            GtkTreeModel      *model,
+                            GtkTreeIter       *iter,
+                            gpointer           data)
+{
+  gint column_id;
+  guint32 time;
+  gchar *readable_time;
+
+  column_id = GPOINTER_TO_INT (data);
+
+  gtk_tree_model_get(model, iter, column_id, &time, -1); 
+  readable_time = format_date_for_display (time);
+  
+  g_object_set(cell, "text", readable_time, NULL);
+
+  g_free (readable_time);
+}
+
+/***************************************************
+ * Functions for sorting the tree views
+ */
+
+gint
+tree_model_compare_strings (GtkTreeModel *model,
+			    GtkTreeIter  *a,
+			    GtkTreeIter  *b,
+			    gpointer      data)
+{
+  gint   column;
+  gint   ret;
+  gchar *string1;
+  gchar *string2;
+
+  column = GPOINTER_TO_INT (data);
+
+  gtk_tree_model_get (model, a, column, &string1, -1);
+  gtk_tree_model_get (model, b, column, &string2, -1);
+
+  ret = g_utf8_collate (string1, string2);
+
+  g_free (string1);
+  g_free (string2);
+
+  return ret;
+}
+
+gint
+tree_model_compare_uints (GtkTreeModel *model,
+			  GtkTreeIter  *a,
+			  GtkTreeIter  *b,
+			  gpointer      data)
+{
+  gint  column;
+  guint number1;
+  guint number2;
+
+  column = GPOINTER_TO_INT (data);
+
+  gtk_tree_model_get (model, a, column, &number1, -1);
+  gtk_tree_model_get (model, b, column, &number2, -1);
+
+  if (number1 != number2)
+    {
+      return (number1 > number2) ? 1 : -1;
+    }
+  else
+    {
+      return 0;
+    }
+}
